#include "entropy.h"

double entropy::count(const std::string& data)
{
	return count(data.data(), data.length());
}

double entropy::count(const char* data, std::size_t length)
{
	double bytes[256] = {0};
	double total = 0.0;
	for(unsigned int i = 0; i < length; i++)
		bytes[data[i]] += 1;

	for(unsigned int i = 0; i < 256; i++)
		if(bytes[i])
			total += abs(log(bytes[i] / length) / log(2.0)) * bytes[i];

	return total;
}

double entropy::count(std::istream& data, std::size_t* length)
{
	double bytes[256] = {0};
	double total = 0.0;
	unsigned int len = 0;
	while(!data.eof())
	{
		len++;
		bytes[static_cast<unsigned char>(data.get())] += 1;
	}

	for(unsigned int i = 0; i < 256; i++)
		if(bytes[i])
			total += abs(log(bytes[i] / len) / log(2.0)) * bytes[i];

	if(length)
		*length = len;

	return total;
}

double entropy::possible_compression(const char* data, std::size_t length)
{
	return count(data, length) / (length * 8);
}

double entropy::possible_compression(const std::string& data)
{
	return possible_compression(data.data(), data.length());
}

double entropy::possible_compression(std::istream& data)
{
	std::size_t length = 0;
	return count(data, &length) / (length * 8);
}
